﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

using System.IO.Ports;

namespace Sample_UART {
	public partial class MainForm : Form {
		public MainForm() {
			InitializeComponent();
		}

		/* 起動イニシャライズ */
		private void MainForm_Load(object sender, EventArgs e) {
			btEnableChange(false);
			comLoad();
		}

		/* UART接続状態によってボタンやテキストボックスの状態変更 */
		private void btEnableChange(bool _serialPortIsOpen) {
			cB_COM.Enabled = !_serialPortIsOpen;
			bt_COMconnect.Enabled = !_serialPortIsOpen;
			bt_COMrecognition.Enabled = !_serialPortIsOpen;
			bt_COMdisconnect.Enabled = _serialPortIsOpen;
			tB_TX.Enabled = _serialPortIsOpen;
			bt_TX.Enabled = _serialPortIsOpen;
		}

		/* UART COM再認識 */
		private void bt_COMrecognition_Click(object sender, EventArgs e) {
			comLoad();
		}

		/* UART COM検索 */
		private void comLoad() {
			cB_COM.Items.Clear();

			string[] _ports = SerialPort.GetPortNames();
			foreach (string _port in _ports) {
				cB_COM.Items.Add(_port);
			}

			if (cB_COM.Items.Count > 0) {
				cB_COM.SelectedIndex = 0;
			}
		}

		/* UART COM接続 */
		private void bt_COMconnect_Click(object sender, EventArgs e) {
			if (!serialPort.IsOpen) {
				serialPort.PortName = cB_COM.Text;
				serialPort.BaudRate = 38400;
				serialPort.Parity = Parity.None;
				serialPort.DataBits = 8;
				serialPort.StopBits = StopBits.One;
				serialPort.Handshake = Handshake.None;
				serialPort.DtrEnable = true;
				serialPort.RtsEnable = true;
				serialPort.WriteTimeout = 500;
				serialPort.ReadTimeout = 500;
				serialPort.Open();

				btEnableChange(serialPort.IsOpen);
			}
		}

		/* UART COM切断 */
		private void bt_COMdisconnect_Click(object sender, EventArgs e) {
			if (serialPort.IsOpen) {
				serialPort.Close();

				btEnableChange(serialPort.IsOpen);
			}
		}

		/* UART 送信 */
		private void bt_TX_Click(object sender, EventArgs e) {
			if (serialPort.IsOpen) {
				serialPort.Write(tB_TX.Text + "\r");
			}
		}

		/* UART 受信 */
		private void serialPort_DataReceived(object sender, SerialDataReceivedEventArgs e) {
			string _rxData = serialPort.ReadExisting();
			serialPort_response(_rxData);
		}

		/* UART 受信データ テキスト表示 */
		delegate void serialPort_setTextCallBack(string text);
		private void serialPort_response(string text) {
			if (tB_RX.InvokeRequired) {
				serialPort_setTextCallBack _stcb = new serialPort_setTextCallBack(serialPort_response);
				Invoke(_stcb, new object[] { text });
			} else {
				tB_RX.Text = text;
			}
		}
	}
}
